% Test performance of TSC and SSC for clustering of randomly projected face
% images
%
% This function was used to obtain the curves in Figure 2 in
% ''Subspace clustering of dimensionality-reduced data'' by Reinhard
% Heckel, Michael Tschannen and Helmut Boelcskei, 2014
%
% Arguments
% algo: 1-SSC; 2-TSC


function [] = face_clustering(algo, red_sim)

    addpath ../include
    addpath ../SSC_ADMM_v1.1
    addpath ../TSC

    s = RandStream('mcg16807','Seed',200);
    RandStream.setGlobalStream(s);

    % Load struct "data" containing downsampled face images from the 
    % Extended Yale B dataset (size: 96 x 84 pixels)
    load 'YaleBCrop05.mat'

    % Load indices of test images
    load 'test_sets.mat'
    
    if (nargin < 1) || (algo == 1)
        % lambda for SSC
        params = [0.001 0.002 0.004 0.008 0.01 0.02 0.04 0.08 0.1 0.2 0.4];
        SCfunc = @SSCRP;
        SCname = 'SSC';
    else
        % q for TSC
        params = 2:2:20;
        SCfunc = @TSCRP;
        SCname = 'TSC';
    end
        
    if (nargin >= 2) && (red_sim == true)
        % Number of examples considered for each number of subspaces
        nrep = 20;
        % Number clustering repetitions for independently generated Phi
        phirep = 5;
        % Considered numbers of subspaces
        nbr_subs = [2 3 4];
        % Projection dimensions to be tested
        ps = 80:80:400;
    else
        nrep = 100;
        phirep = 5;
        nbr_subs = [2 3 4];
        ps = 20:20:400;
    end

    missrates = zeros(length(nbr_subs),length(ps),length(params),nrep);
    fdes = missrates;
    tcalcs = missrates;

    % Test all configurations
    for i = 1:length(nbr_subs)
        curr_set = test_sets{i};
        for j = 1:nrep
            
            fprintf(['Nbr of subspaces: ' num2str(nbr_subs(i)) ', Example Nbr: ' num2str(j) '\n'])
            
            sub_idx = curr_set(j,:);

            Ytot = [];
            GT = [];
            for k = 1:length(sub_idx)
                Ycurr = double(data{sub_idx(k)});
                Ytot = [Ytot normc(Ycurr)];
                GT = [GT k*ones(1,size(Ycurr,2))];
            end

            for k = 1:length(ps)
                for m = 1:phirep
                    tic
                    Phi = 1/sqrt(ps(k))*randn(ps(k), size(Ytot,1));
                    tphi = toc;

                    for l = 1:length(params);
                        
                        [missr, fde, tcalc] = SCfunc(Ytot,params(l),GT,nbr_subs(i),Phi);

                        missrates(i,k,l,j) = missrates(i,k,l,j) + missr/phirep;
                        fdes(i,k,l,j) = fdes(i,k,l,j) + fde/phirep;
                        tcalcs(i,k,l,j) = tcalcs(i,k,l,j) + (tcalc+tphi)/phirep;
                    end
                end
            end
        end
    end

    missmeans = nanmean(missrates,4);
    tcalcmeans = nanmean(tcalcs,4);

    missmeans_opt = zeros(length(nbr_subs),length(ps));
    params_opt = missmeans_opt;
    tcalcs_opt = missmeans_opt;

    % Select param yielding the lowest CE
    for i = 1:length(nbr_subs)
        [missmeans_opt(i,:), idx] = min(squeeze(missmeans(i,:,:)),[],2);
        for j = 1:length(ps)
            params_opt(i,j) = params(idx(j));
            tcalcs_opt(i,j) = tcalcmeans(i,j,idx(j));
        end
    end

    fprintf(['Algorithm: ' SCname '\n']);
    fprintf('Mean missrates for optimal param (rows: nbr of subjects, columns: p) \n');
    disp(missmeans_opt);
    fprintf('Optimal param values (rows: nbr of subjects, columns: p) \n');
    disp(params_opt);
    fprintf('Mean computation times for optimal param (rows:  nbr of subjects, columns: p) [s] \n');
    disp(tcalcs_opt);

    for i = 1:size(missmeans_opt,1)
        dlmwrite(['./CE_FaceClustering_' SCname '_' num2str(nbr_subs(i)) 'F.dat'], [ps',missmeans_opt(i,:)'],'delimiter',' ');
    end

    save(['RP' SCname 'Faces.mat'],'nrep','nbr_subs','params','ps','missrates','fdes','tcalcs','missmeans_opt','params_opt','tcalcs_opt','phirep')
end