% Wrapper for the LASSO-SSC ADMM implementation by Ehsan Elhamifar
%
% Arguments
% Y:      Data matrix
% lambda: LASSO regularization parameter
% gt:     Vector containing the data labels
% L:      Nbr of subspaces
% p:      Projection dimension or projection matrix
% tol:    Optimization objective value tolerance
% iter:   Maximum number of iterations

function [missr, fde, tcalc] = SSCRP(Y, lambda, gt, L, p, tol, iter)

    narginchk(4,7);
    if nargin < 5, p = 0; end
    if nargin < 6, tol = 1e-4; end
    if nargin < 7, iter = 200; end

    % Compensate for data dependent normalization in the ADMM
    % implementation
    alpha(1) = computeLambda_mat(Y)/lambda;
    alpha(2) = 1/lambda;
    
    try
        tic

        % Project data
        if(sum(size(p)) > 2)
            Y = p*Y;
        elseif p > 0
            Phi = 1/sqrt(p)*randn(p, size(Y,1));
            Y = Phi*Y;
        end

        % LASSO-SSC
        Z = admmLasso_mat_func(Y,false,alpha,tol,iter);

        Zaff = BuildAdjacency(Z);

        grp = SpectralClustering(Zaff,L);

        tcalc = toc;

        missr = Misclassification(grp,gt);

        fde = feature_detection_error_l1(Z,L);
        
    catch err
        fprintf(['ERROR: ' err.message '\n']);
        
        save(['SSCdebug' datestr(now,'ddmmyyyyHHMMSSFFF') '.mat'],'Z','Zaff','Y','lambda','gt','L','p');
        
        tcalc = NaN;
        missr = NaN;
        fde = NaN;
    end
end

