% Test performance of SSC or TSC for randomly projected synthetic data
% laying in subspaces with intersections
% 
% This function was used to obtain the curves in Figure 1 in
% ''Subspace clustering of dimensionality-reduced data'' by Reinhard
% Heckel, Michael Tschannen and Helmut Boelcskei, 2014
%
% Arguments
% algo: 1-SSC; 2-TSC

function [] = subspace_intersections(algo,red_sim)

    addpath ../include
    addpath ../SSC_ADMM_v1.1
    addpath ../TSC

    s = RandStream('mcg16807','Seed',200);
    RandStream.setGlobalStream(s);
    
    if (nargin < 1) || (algo == 1)
        % lambda
        params = [0.001 0.002 0.004 0.008 0.01 0.02 0.04 0.08 0.1 0.2 0.4];
        SCfunc = @SSCRP;
        SCname = 'SSC';
    else
        % q
        params = 2:2:20;
        SCfunc = @TSCRP;
        SCname = 'TSC';
    end
    
    % Subspace dimensions
    ds = [20 30];
    nbr_subs = 5;
    % Ambient dimension
    n = 200;
    % Intersection ratios
    int_rat = 0.2;
    % Sampling density
    rho = 5;

    if (nargin >= 2) && (red_sim == true)
        % Projection dimensions
        ps = 10:10:80;
        % Number of problem instances
        nrep = 20;
    else
        ps = 4:2:80;
        nrep = 50;
    end
    

    % Sweep all configurations and record performance
    missrates = zeros(length(ds),length(ps),length(params),nrep);
    fdes = missrates;
    tcalcs = missrates;
    for i = 1:length(ds);
        for j = 1:nrep
            
            fprintf(['Dimension: ' num2str(ds(i)) ', Example Nbr: ' num2str(j) '\n'])
            
            for k = 1:length(ps)

                [Y, GT] = generate_intersection_data(n,ds(i)*ones(1,nbr_subs),int_rat,rho);
                
                % Generate Phi here to ensure same setting for each
                % parameter
                tic
                Phi = 1/sqrt(ps(k))*randn(ps(k), size(Y,1));
                tphi = toc;

                for l = 1:length(params);
                    [missr, fde, tcalc] = SCfunc(Y,params(l),GT,nbr_subs,Phi);

                    missrates(i,k,l,j) = missr;
                    fdes(i,k,l,j) = fde;
                    tcalcs(i,k,l,j) = tcalc + tphi;
                end
            end 
        end
    end

    missmeans = mean(missrates,4);
    tcalcmeans = mean(tcalcs,4);
    
    % Select optimum parameter value
    missmeans_opt = zeros(length(ds),length(ps));
    params_opt = missmeans_opt;
    tcalcs_opt = missmeans_opt;

    for i = 1:length(ds)
        [missmeans_opt(i,:), idx] = min(squeeze(missmeans(i,:,:)),[],2);
        for j = 1:length(ps)
            params_opt(i,j) = params(idx(j));
            tcalcs_opt(i,j) = tcalcmeans(i,j,idx(j));
        end
    end

    for i = 1:size(missmeans_opt,1)
        dlmwrite(['./CE_SubInt_' SCname '_' num2str(i) '.dat'], [ps',missmeans_opt(i,:)'],'delimiter',' ');
    end

    fprintf(['Algorithm: ' SCname '\n']);
    fprintf('Mean missrates for optimal param (rows: intersection ratio, columns: p) \n');
    disp(missmeans_opt);
    fprintf('Optimal param values (rows: intersection ratio, columns: p) \n');
    disp(params_opt);
    fprintf('Mean computation times for optimal param (rows: intersection ratio, columns: p) [s] \n');
    disp(tcalcs_opt);

    save(['RPSubspaceIntersection' SCname '.mat'],'ds','n','ps','nrep','int_rat','rho','params','missrates','fdes','tcalcs','missmeans_opt','params_opt','tcalcs_opt')
end